import { ForbiddenException, Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { PassportStrategy } from '@nestjs/passport';
import { Strategy, ExtractJwt } from 'passport-jwt';
import { ModuloService } from '../../modulo/modulo.service';
import { OperadorService } from '../../operador/operador.service';
import { UsuarioService } from '../../usuario/usuario.service';
import { Modulo } from '../../modulo/entity/modulo.entity';
import { Operador } from '../../operador/entity/operador.entity';
import { Usuario } from '../../usuario/entity/usuario.entity';
import { JwtPayloadAdmin } from '../dto/jwt-payload-admin';
import { JwtPayloadOperador } from '../dto/jwt-payload-operador';
import { JwtPayloadUsuario } from '../dto/jwt-payload-usuario';

@Injectable()
export class JwtStrategyService extends PassportStrategy(Strategy) {
  constructor(
    private configService: ConfigService,
    private moduloService: ModuloService,
    private operadorService: OperadorService,
    private usuarioService: UsuarioService,
  ) {
    super({
      // Configuración de JWT
      jwtFromRequest: ExtractJwt.fromAuthHeaderAsBearerToken(),
      secretOrKey: configService.get<string>('AUTH_SECRETKEY'),
    });
  }

  // Función que se ejecuta despues de aceptar un token valido
  async validate(
    // Payload del JWT que mandó el client
    payload: JwtPayloadAdmin | JwtPayloadOperador | JwtPayloadUsuario,
  ) {
    const user: { modulo?: Modulo; usuario?: Usuario; operador?: Operador } =
      {};

    // Si el token contiene un objeto Usuario en el
    if ('Usuario' in payload)
      // Buscar a ese usuario en la db y lo asigna al objeto user
      await this.usuarioService
        .findInfoUsuarioById(payload.Usuario.id_usuario)
        .then((usuario) => (user.usuario = usuario));
    // Si el token contiene un objeto Operador en el
    else if ('Operador' in payload)
      // Buscar a ese operador en la db y lo asigna al objeto user
      await this.operadorService
        .findInfoOperadorById(payload.Operador.id_operador)
        .then(async (operador) => {
          // Valida que la cuena del operador este activa
          if (!operador.activo)
            throw new ForbiddenException('Esta cuenta esta desactivada.');
          // Validar que venga en el payload venga con un objeto Modulo en
          // caso de ser tipo usuario operador y asignarlo al objeto user
          if (operador.tipoUsuario.id_tipo_usuario === 4) {
            if ('Modulo' in payload)
              await this.moduloService
                .findInfoModuloById(payload.Modulo.id_modulo)
                .then((modulo) => (user.modulo = modulo));
            else
              throw new ForbiddenException(
                'Credenciales no válidas, inicia sesión de nuevo.',
              );
          }
          user.operador = operador;
        });
    // Este objeto retornado se encuentra en el objeto request
    // (El ojbeto que recibe toda la información de la petición)
    return user;
  }
}
