import { ConflictException, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Modelo } from './entity/modelo.entity';

@Injectable()
export class ModeloService {
  constructor(
    @InjectRepository(Modelo) private repository: Repository<Modelo>,
  ) {}

  create(modelo: string, tipo: string) {
    return this.findByModelo(modelo, tipo, false)
      .then((existeModelo) => {
        if (existeModelo) throw new ConflictException('Ya existe este modelo.');
        return this.repository.save(this.repository.create({ modelo, tipo }));
      })
      .then((_) => ({
        message: 'Se creó correctamente un nuevo modelo.',
      }));
  }

  findAll(tipo: string) {
    return this.repository.find({ where: { tipo }, order: { tipo: 'ASC' } });
  }

  findById(id_modelo: number, tipo: string, validarExistencia = true) {
    return this.repository
      .findOne({ where: { id_modelo, tipo } })
      .then((modelo) => {
        if (validarExistencia && !modelo)
          throw new ConflictException('No existe este modelo.');
        return modelo;
      });
  }
  findByModelo(modelo: string, tipo: string, validarExistencia = true) {
    return this.repository
      .findOne({ where: { modelo, tipo } })
      .then((modelo) => {
        if (validarExistencia && !modelo)
          throw new ConflictException('No existe este modelo.');
        return modelo;
      });
  }
}
